package com.textura.cpms.web.addressbook.data

import java.beans.PropertyDescriptor
import java.util.Map

import org.slf4j.Logger
import org.slf4j.LoggerFactory

import com.textura.cpms.model.Person
import com.textura.cpms.model.PersonReference
import com.textura.cpms.service.PersonManager
import com.textura.cpms.service.QueryMetaData
import com.vaadin.data.Container
import com.vaadin.data.Item
import com.vaadin.data.Property
import com.vaadin.data.Container.ItemSetChangeEvent
import com.vaadin.data.Container.ItemSetChangeListener
import com.vaadin.data.util.ObjectProperty
import com.vaadin.data.util.PropertysetItem

class PersonReferenceContainer implements Container, Container.ItemSetChangeNotifier {

	static final Logger logger = LoggerFactory.getLogger(PersonReferenceContainer)
	
	/**
	 * Natural property order for Person bean. Used in tables and forms.
	 */
	public static final Object[] NATURAL_COL_ORDER = [
		'firstName',
		'lastName',
		'email',
		'phoneNumber',
		'streetAddress',
		'postalCode',
		'city'
	]

	protected static final Collection<Object> NATURAL_COL_ORDER_COLL = Collections.unmodifiableList(Arrays.asList(NATURAL_COL_ORDER))
	
	/**
	 * "Human readable" captions for properties in same order as in NATURAL_COL_ORDER.
	 */
	public static final String[] COL_HEADERS_ENGLISH = [
		'First name',
		'Last name',
		'Email',
		'Phone number',
		'Street Address',
		'Postal Code',
		'City'
	]
	
	/**
	 * Default query meta data.
	 */
	public static QueryMetaData defaultQueryMetaData = new QueryMetaData(['firstName', 'lastName'] as String[], [true, true] as boolean[])

	protected final PersonManager personManager
	protected List<Item> personReferences
	protected Map<Object, Item> idIndex
	protected QueryMetaData queryMetaData = defaultQueryMetaData

	List<ItemSetChangeListener> listeners = []

	PersonReferenceContainer(PersonManager personManager) {
		this.personManager = personManager
	}

	QueryMetaData getQueryMetaData() {
		return queryMetaData
	}
	
	protected setQueryMetaData(QueryMetaData queryMetaData) { this.queryMetaData = queryMetaData }

	void close() {
		if (personReferences != null) {
			personReferences.clear()
			personReferences = null
		}
	}

	boolean isOpen() {
		return personReferences != null
	}
	
	List<PersonReference> getItems() {
		return personReferences
	}
	
	void refresh() {
		refresh(queryMetaData)
	}

	void refresh(QueryMetaData queryMetaData) {
		this.queryMetaData = queryMetaData
		(personReferences, idIndex) = load() 
		notifyListeners()
	}
	
	
	@Override
	synchronized void addListener(ItemSetChangeListener listener) {
		listeners.add(listener)
	}

	@Override
	synchronized void removeListener(ItemSetChangeListener listener) {
		listeners.remove(listener)
	}
	
	@Override
	boolean containsId(Object itemId) {
		return idIndex.containsKey(itemId)
	}


	@Override
	Item getItem(Object itemId) {
        return idIndex[itemId]	
	}

	@Override
	Collection<?> getContainerPropertyIds() {
		return NATURAL_COL_ORDER_COLL
	}

	@Override
	Collection<?> getItemIds() {
		return Collections.unmodifiableSet(idIndex.keySet())
	}

	@Override
	Property getContainerProperty(Object itemId, Object propertyId) {
		Item item = idIndex.get(itemId)
		if (item != null) {
			return item.getItemProperty(propertyId)
		}
		return null
	}

	@Override
	Class<?> getType(Object propertyId) {
        try {
            // TODO Optimize, please!
            PropertyDescriptor pd = new PropertyDescriptor((String) propertyId, Person.class)
            return pd.getPropertyType()
        } 
		catch (Exception e) {
            return null
        }
	}
	
	@Override
	int size() {
		return personReferences == null ? 0 : personReferences.size()
	}

	@Override
	Item addItem(Object itemId) throws UnsupportedOperationException {
		throw new UnsupportedOperationException('This container is read-only.')
	}

	@Override
	Object addItem() throws UnsupportedOperationException {
		throw new UnsupportedOperationException('This container is read-only.')
	}

	@Override
	boolean removeItem(Object itemId) throws UnsupportedOperationException {
		throw new UnsupportedOperationException('This container is read-only.')
	}

	@Override
	boolean addContainerProperty(Object propertyId, Class<?> type, Object defaultValue) throws UnsupportedOperationException {
		throw new UnsupportedOperationException('This container is read-only.')
	}

	@Override
	boolean removeContainerProperty(Object propertyId)	throws UnsupportedOperationException {
		throw new UnsupportedOperationException('This container is read-only.')
	}

	@Override
	boolean removeAllItems() throws UnsupportedOperationException {
		throw new UnsupportedOperationException('This container is read-only.')
	}
	
	protected def load() {
		def references = personManager.findPersonReferences(queryMetaData, NATURAL_COL_ORDER as String[])
		
		def items = []
		def idMap = [:]
		
		references.each { person ->
			def item = new PropertysetItem()
			person.propertyMap.each { k, v ->
				item.addItemProperty(k, new ObjectProperty(v, getType(k)))
			}
			items << item
			idMap[person.personId] = item
		}
		
		logger.debug('items = {}', items)
		
		return [items.asImmutable(), idMap.asImmutable()]
	}

	protected void notifyListeners() {
		final List<ItemSetChangeListener> cl
		synchronized(this) {
			cl = listeners.clone()
		}
		ItemSetChangeEvent event = new ItemSetChangeEvent() {
			public Container getContainer() {
				return PersonReferenceContainer.this
			}
		}
		cl.each { listener ->
			listener.containerItemSetChange(event)
		}
	}
}
